package gxpconvert;

import gpxconvert.jaxb.gpx.GpxType;
import gpxconvert.jaxb.gpx.RteType;
import gpxconvert.jaxb.gpx.WptType;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JTextArea;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;

public class ConvertGpx implements GPXConverterI {
	private GpxType gpxType;
	private JTextArea outputPanel;

	public static void main(String[] args) {
		ConvertGpx convertGpx = new ConvertGpx();
		convertGpx.buildGUI();

	}

	// =========================================================================
	private void buildGUI() {
		JFrame frame = new MainFrame("GPX-Convert tool for OruxMaps");
		outputPanel = new JTextArea();
		frame.add(outputPanel);
		outputPanel.setTransferHandler(new DropTranspherHandler(this));
		outputPanel.append("The tool does:\n" + "Extract waypoints from a route\n" + "Copy the waypoint content of cmt to the name.\n"
				+ "Add these changed waypoints as extra waypoints and delete the route.\n" + "Creates a new file with the new gpx content\n"+ "\nDrop gpx files to be converted here.\n\n");
		frame.setSize(600, 400);
		frame.setVisible(true);
	}

	// =========================================================================

	public boolean convertgpxFile(File pSrcFile, File pDestFile) {
		boolean result = false;
		try {
			GenericXMLHelper genericXMLHelper = new GenericXMLHelper();
			try {
				JAXBElement<?> jaxBElement = genericXMLHelper.createObject(new FileInputStream(pSrcFile));

				gpxType = (GpxType) jaxBElement.getValue();

				convertJaxbObject(gpxType);
				OutputStream destStream = new FileOutputStream(pDestFile);
				genericXMLHelper.createXMLData(jaxBElement, destStream, "UTF-8");

				destStream.close();
				outputPanel.append("Converted: " + pSrcFile.getAbsolutePath() + " => " + pDestFile.getAbsolutePath() + "\n");
				result = true;
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		} catch (JAXBException e) {
			e.printStackTrace();
		}
		return result;
	}

	// =========================================================================

	private void convertJaxbObject(GpxType pGpxType) {
		RteType rteType = null;
		for (Object o : pGpxType.getMetadataOrWptOrRte()) {
			if (o instanceof RteType) {
				rteType = (RteType) o;
				convertRtepts(rteType);
				break;
			}
		}
		pGpxType.getMetadataOrWptOrRte().remove(rteType);
	}

	private void convertRtepts(RteType pRteType) {
		// WptType wptType = new WptType();
		List<Object> wptList = new ArrayList<Object>();
		for (WptType wptType : pRteType.getRtept()) {
			// System.out.println("ja");
			String wegbeschreibung = wptType.getCmt();
			wptType.setType(wegbeschreibung);
			wptType.setName(wegbeschreibung);
			wptList.add(wptType);
		}
		gpxType.getMetadataOrWptOrRte().addAll(0, wptList);
	}

}
