package gxpconvert;

import gpxconvert.jaxb.gpx.ObjectFactory;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.apache.log4j.Logger;
import org.xml.sax.SAXException;

public class GenericXMLHelper {
	private static Logger logger = Logger.getLogger(GenericXMLHelper.class);
	private Unmarshaller unmarshaller = null;
	private Marshaller marshaller;
	private JAXBContext jaxbContext;
	private String schemaFile;
	private String contentPath;
	private Class<?> objectFactory;
	private Schema schema;

	public GenericXMLHelper() {
		schemaFile = "/resources/gpx.xsd";
		contentPath = "gpxconvert.jaxb.gpx";
		objectFactory = ObjectFactory.class;
	}

	public Class<?> getContentObject() {
		return objectFactory;
	}

	private JAXBContext getJAXBContext() throws JAXBException {
		if (jaxbContext == null) {
			jaxbContext = JAXBContext.newInstance(contentPath);
		}
		return jaxbContext;
	}

	// ==========================================================================

	private Schema getSchema() {
		if (schema == null) {
			SchemaFactory sf = SchemaFactory.newInstance(javax.xml.XMLConstants.W3C_XML_SCHEMA_NS_URI);
			try {
				URL schemaURL = GenericXMLHelper.class.getResource(schemaFile);
				schema = sf.newSchema(schemaURL);
			} catch (SAXException e) {
				logger.error("getUnmarshaller() not able to read schema file: " + schemaFile);
				throw new RuntimeException(e);
			}
		}
		return schema;
	}

	// ==========================================================================

	private Marshaller getMarshaller() throws JAXBException {
		if (marshaller == null) {
			marshaller = getJAXBContext().createMarshaller();
			marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
			marshaller.setSchema(getSchema());
		}
		return marshaller;
	}

	// ==========================================================================

	private Unmarshaller getUnmarshaller() throws JAXBException {
		if (unmarshaller == null) {
			unmarshaller = getJAXBContext().createUnmarshaller();
			unmarshaller.setSchema(getSchema());
		}
		return unmarshaller;
	}

	// ==========================================================================
	/**
	 * Has to be synchronized, because we are setting the encoding for the
	 * global marshaller.
	 */

	public synchronized void createXMLData(Object element, OutputStream pOutput, String pEncoding) throws JAXBException {
		logger.info("createXMLData() start");
		try {
			Marshaller myMarshaller = getMarshaller();
			if (pEncoding != null && pEncoding.trim().length() > 0) {
				myMarshaller.setProperty(Marshaller.JAXB_ENCODING, pEncoding);
			}
			myMarshaller.marshal(element, pOutput);
			logger.info("createXMLData() xml done");
		} catch (JAXBException e) {
			logger.error("createXMLData() error while writing xml data", e);
			throw e;
		}
	}

	// ==========================================================================

	public JAXBElement<?> createObject(InputStream pInput) throws JAXBException {
		JAXBElement<?> result = null;
		try {
			if (pInput != null) {
				result = (JAXBElement<?>) getUnmarshaller().unmarshal(pInput);
				pInput.close();
			}
		} catch (JAXBException e) {
			// logger.debug("Data not readable or not matching the schema",
			// e);
			throw e;
		} catch (IOException e) {
			logger.error("createObject() can not close input stream", e);
			throw new RuntimeException(e);
		}
		return result;
	}

	// ==========================================================================

}
